function trace = kpcfit_init(S, varargin)
% trace = kpcfit_init(S,'option1','val1','option2','val2',...)
%
% DESCRIPTION
% Prepare trace S for KPC fitting
%
% OPTION LIST
% 'ACLags'     - autocorrelation lags to be considered
% 'BCGridLags' - lags defining bicovariance square grid to be considered

%% options
OptionNames = [
    'ACLags    ';
    'BCGridLags';
    ];

OptionTypes = [
    'numeric';
    'numeric'];

OptionValues = [];
for i = 1:size(OptionNames,1)
    options.(deblank(OptionNames(i,:)))=[];
end

% Default settings
n = length(S);
options.MaxMoment = 3; % fit 3 moments (this value is not currently used)
options.ACLags = unique(logspacei(1,ceil(length(S)/10),500)); % maximum lags of AC used for fitting
options.BCGridLags = unique(logspacei(1,max(options.ACLags),5)); % lags used to generate square BC grid
% Parse optional parameters
options=ParseOptPara(options,OptionNames,OptionTypes,OptionValues,varargin);

%% trace preprocessing
fprintf(1,'init: computing moments from the trace\n');
% moments
E=zeros(1,options.MaxMoment);
for j=1:options.MaxMoment
    E(j)=mean(S.^j);
end

fprintf(1,'init: computing autocorrelations from the trace\n');
% autocorrelations
AC=trace_acf(S,options.ACLags);
ACFull=trace_acf(S,1:length(S)/10);

% estimate cut point of AC
posmax = options.ACLags(min(find(abs(AC) < 10^-6)));
%fprintf(1,'init: autocorrelations beyond lag %d ignored (abs. val. <10^-6)\n',posmax);
if ~isempty(posmax)    
    todel = find(options.ACLags>posmax);
    options.ACLags(todel)=[]; % lags of AC used for fitting
    AC(todel)=[];
    todel = find(options.BCGridLags>posmax);
    options.BCGridLags(todel)=[]; % lags of AC used for fitting
end

fprintf(1,'init: computing bicovariances from the trace\n');
[BC,BCLags]=trace_bicov(S,options.BCGridLags); % bispectrum coefficients

%% generate trace structure
trace = struct('S',S(:),'E',E,'AC',AC(:),'ACFull',ACFull(:),'ACLags',options.ACLags,'BC',BC,'BCGridLags',options.BCGridLags,'BCLags',BCLags);
end
