function [PH,subopt] = kpcfit_ph_auto(E,varargin)
%  ** beta version **
%
%  PH = kpcfit_ph_auto(E)
%  [PH,suboptimalPHs] = kpcfit_ph_auto(E)
%
% DESCRIPTION
% Fit phase-type (PH) process using Kronecker Product Composition (KPC)
% method using extended search.
%
% INPUT
%
% E              - vector of moments of consecutive order to be fitted
%
% OUTPUT
%
% PH             - PH distribution that best matches moments
% suboptPHs      - other sub-optimal PH distributions evaluated in the fitting
%
% OPTIONS
%
% 'MaxNumStates' - integer, maximum number of states in PH model - ** KPC returns only multiples of 2 **, default: 32
% 'MinOrdMom'    - integer, minimum number of moments to be fitted (best-effort)
% 'MaxOrdMom'    - integer, maximum number of moments to be fitted (best-effort)
% 'MaxRunsMM'    - integer, maximum number of runs for each individual choice of number of states and moments
%
% EXAMPLE
%
% S; % trace
% E = [];
% for k=1:6 E(k)=mean(S.^k); end
% PH=kpcfit_ph_auto(E,'MinOrdMom',2,'MaxOrdMom',4,'MaxRunsMM',1)
% [E;map_moment(PH,1:6)]'

OptionNames = [
    'WantHyper   ';
    'MinNumStates';
    'MaxNumStates';
    'MinOrdMom   ';
    'MaxOrdMom   ';
    'MaxRunsMM   ';
    ];

OptionTypes = [
    'numeric';
    'numeric';
    'numeric';
    'numeric';
    'numeric';
    'numeric'];

OptionValues = [];
for i = 1:size(OptionNames,1)
    options.(deblank(OptionNames(i,:)))=[];
end

% Default settings
options.WantHyper = 0;
options.MinNumStates = 2;
options.MaxNumStates = 32;
options.MinOrdMom = 2;
options.MaxOrdMom = length(E);
options.MaxRunsMM = 5;

% Parse optional parameters
options=ParseOptPara(options,OptionNames,OptionTypes,OptionValues,varargin);
options.MaxNumStates = max([options.MinNumStates,options.MaxNumStates]);
if (E(2)-E(1)^2)/E(1)^2 >= 1
    options.WantHyper = 1;
else
    options.WantHyper = 0;
end
if options.MinOrdMom < 2
    error('Selected value of MinOrdMom is infeasible, must be integer > 1');
end
if options.MaxOrdMom < 2
    error('Selected value of MaxOrdMom is infeasible, must be integer > 1');
end
if options.MaxRunsMM < 1
    error('Selected value of MaxRunsMM is infeasible, must be integer > 1');
end
if ~(options.WantHyper == 1 | options.WantHyper == 0)
    error('Selected value of WantHyper is infeasible, must be true or false');
end
%%
i = 0;
nPHset=ceil(log2(options.MinNumStates)):ceil(log2(options.MaxNumStates));
nMomset=options.MinOrdMom:options.MaxOrdMom;
subopt = {};

fprintf('kpcfit_ph: type "help kpcfit_ph_auto" for options\n')

for nPH=nPHset
    for nMom=nMomset
        for it=1:options.MaxRunsMM
            i = i + 1;
            tic;
            fprintf('kpcfit_ph: run %d of %d [PH order %d, first %d moments evaluated]',i,length(nPHset)*length(nMomset)*options.MaxRunsMM, 2^nPH, nMom);
            PH=kpcfit_ph_manual(E,'NumPHs',nPH,'NumMoms',nMom,'WantHyper',options.WantHyper);
            if map_isfeasible(PH)>=10
                fprintf(' (%0.3f sec) *** feasible\n',toc);
            else
                fprintf(' (%0.3f sec)\n',toc);
            end
            subopt{i}=PH;
            delta(i)=norm(map_moment(PH,1:length(E))-E);
        end
    end
end

I=[];
for i=1:length(subopt)
    I(end+1,:)=([i,norm(E-map_moment(subopt{i},1:length(E)))]);
end
[a,b]=sort(I(:,2));

best = 1;
while map_isfeasible(subopt{b(best)})<10
    if best == length(subopt)
        break;
    end
    best = best + 1;
end

PH = map_normalize(subopt{b(best)});
if map_isfeasible(subopt{b(best)})<10    
    if isnan(PH{1}(1)) | isnan(PH{1}(2)) | min(PH{2}(:))<0 | min(PH{1}(:)-diag(PH{1}(:)))<0
        fprintf('kpcfit_ph: WARNING - no feasible PH distribution found, returning best effort PH(2) or Erlang fitting\n')
        s=(E(2)-E(1)^2)/E(1)^2; %scv
        if s>1
            if length(E)<3
                PH=map_feasblock(E(1),E(2),-1,0);
            else
                PH=map_feasblock(E(1),E(2),E(3),0);
            end
        elseif s<=0.75
            k=minpos(abs(s-1./(1:100))); % match scv
            PH=map_erlang(E(1),k);
        else s>0.75 & s<=1
            PH=map_exponential(E(1));
        end
    end
end
PH=map_scale(PH,E(1));
%% display final moments
disp('** KPC-PH fitting algorithm completed ** ');
disp(' ');
disp('                  Moments Comparison       ');
disp('          Original Trace          Fitted PH');
format long e
for k=1:length(E)
    disp([E(k) map_moment(PH,k)]);
end

end

function MAP=map_erlang(MEAN,k)
% MAP=map_erlang(MEAN,k) - Fit an Erlang-k process as a MAP
%
%  Input:
%  MEAN: mean inter-arrival time of the process
%  k: number of phases
%
%  Output:
%  MAP: a MAP in the form of {D0,D1}
%
%  Examples:
%  - MAP=map_erlang(2,3) an Erlang-3 process with mean 2
%
% MAP Queueing Networks Toolbox
% Version 1.0 	 15-Apr-2008
mu=k/MEAN;

MAP={zeros(k),zeros(k)};
% compute D0
for i=1:k-1
    MAP{1}(i,i+1)=mu;
end
% compute D1
MAP{2}(k,1)=mu;
% normalize D0 diagonal
MAP=map_normalize(MAP);
end

function MAP=map_exponential(MEAN)
% MAP=map_exponential(MEAN) - Fit a Poisson process as a MAP
%
%  Input:
%  MEAN: mean inter-arrival time of the process
%
%  Output:
%  MAP: a MAP in the form of {D0,D1}
%
%  Examples:
%  - MAP=map_exponential(2) return a Poisson process with rate lambda=0.5
%
% MAP Queueing Networks Toolbox
% Version 1.0 	 15-Apr-2008
if iscell(MEAN)
    MAP=map_exponential(map_mean(MEAN));
    return
end
mu=1/MEAN;
MAP={[-mu],[mu]};
end
