function isfeas=map_isfeasible(MAP,TOL)
% map_isfeasible determines if a MAP represented in the form of D0 and D1
% is feasible according to some precision.
%
%  isfeas=map_isfeasible(MAP,TOL) returns a 1 if the input MAP is feasible,
%  and 0 if the input MAP is not feasible.
%   
%       Input: 
%           MAP = a map in the form of {D0,D1}
%           TOL = precision, this is an optional parameter, if no
%                 precision is specified, the function automatically set
%                 precision to 1e-15 to 1e-1 and test if the map is 
%                 feasible according to any of the precision, if so, 
%                 returns true(1), otherwise return 1
%       
%       Output: 
%           isfeasible = 0 or 1, if it is 1, then the MAP is feasible
%           according to the given precision, otherwise the given MAP
%           is not feasible according to the precision


isfeas=0;
if nargin==2
    isfeas=map_checkfeasible(MAP,TOL);
elseif nargin==1
    TOLMAGNITUDE=15;
    for k=TOLMAGNITUDE:-1:1
        check=map_checkfeasible(MAP,10^-k);
        if check
%            if nargout>0
                isfeas=k;
%            end
%            fprintf('MAP is feasible with tolerance 1e-%d\n',k);
            return
        end
    end
    if nargout>0
        isfeas=0;
    end
%    fprintf('MAP is infeasible\n');
end
end

function isfeas=map_checkfeasible(MAP,TOL)
if nargin==1
    TOL=eps;
end
n=size(MAP{1});
D0=MAP{1};
D1=MAP{2};
if abs(det(D0))<1e-10
   warning('D0 determinant <1e-10 ');
   
end
if sum(isnan(D0(:)))>0
    % if any of the elements of D0 is nan
    isfeas=0;
    return
end
P=inv(-D0)*D1;
Q=D0+D1;

% set very small values to zero
D0(find(abs(D0)<TOL))=0;
D1(find(abs(D1)<TOL))=0;
P(find(abs(P)<TOL))=0;
Q(find(abs(Q)<TOL))=0;

% assume initially the MAP is feasible
isfeas=1;
for i=1:n
    for j=1:n
        if i~=j && D0(i,j)<0
            isfeas=0;
        end
        if i==j && D0(i,j)>0
            isfeas=0;
        end
        if D1(i,j)<0
            isfeas=0;
        end
        if i~=j && Q(i,j)<0
            isfeas=0;
        end
        if i==j && Q(i,j)>0
            isfeas=0;
        end
        if P(i,j)<0
            isfeas=0;
        end
    end
    if abs(sum(P(i,:)))<1-n*TOL
        isfeas=0;
    end
    if abs(sum(P(i,:)))>1+n*TOL
        isfeas=0;
    end
    if abs(sum(Q(i,:)))<0-n*TOL
        isfeas=0;
    end
    if abs(sum(Q(i,:)))>0+n*TOL
        isfeas=0;
    end
end
end